@@ -1,4 +1,5 @@
 // RUN: %clang_cc1 -fsyntax-only %s -verify
+// RUN: %clang_cc1 -fsyntax-only %s -fexperimental-new-constant-interpreter -verify
 // expected-no-diagnostics
 
 struct S {
@@ -11,6 +12,6 @@ struct S const foo(void);
 
 struct S tmp;
 
-void priv_sock_init() {
+void priv_sock_init(void) {
   tmp = (struct S)foo();
 }
