@@ -24,7 +24,6 @@
 #include <ranges>
 
 #include "almost_satisfies_types.h"
-#include "boolean_testable.h"
 #include "test_iterators.h"
 
 struct Predicate {
@@ -218,20 +217,6 @@ constexpr bool test() {
     }
   }
 
-  {
-    // check that the implicit conversion to bool works
-    {
-      int a[] = {1, 2, 3, 4};
-      auto ret = std::ranges::find_if_not(a, a + 4, [](const int& i) { return BooleanTestable{i != 3}; });
-      assert(ret == a + 2);
-    }
-    {
-      int a[] = {1, 2, 3, 4};
-      auto ret = std::ranges::find_if_not(a, [](const int& b) { return BooleanTestable{b != 3}; });
-      assert(ret == a + 2);
-    }
-  }
-
   return true;
 }
 
